#!/bin/sh
#
# This script plots the time evolution of the density of
# 1D-problems. The script collects all files ending in x-density.dat
# orders concatenates and plots them.
# 
# You will need a recent version of gnuplot 
# for this script to work (e.g. 4.2.4 is enough)
# 
# Author: Kaspar Sakmann (2007)
# 
OUT_FILE=1d-x-DENSITYDYNAMICS.DAT
PS_FILE=1d-x-DENSITYDYNAMICS.ps
GNUPLOTFILE=1d-x-DENSITYDYNAMICS.GNU
rm -f $GNUPLOTFILE


XDENSITY_FILES=XDENSITY_FILES.SORT

rm -f $OUT_FILE
rm -f $XDENSITY_FILES

ls -t   ?.*x-density.dat     | sort -n > $XDENSITY_FILES
ls -t   ??.*x-density.dat    | sort -n >> $XDENSITY_FILES
ls -t   ???.*x-density.dat   | sort -n >> $XDENSITY_FILES
ls -t   ????.*x-density.dat  | sort -n >> $XDENSITY_FILES
ls -t   ?????.*x-density.dat | sort -n >> $XDENSITY_FILES

FILE_LIST1="`cat $XDENSITY_FILES`"
echo FILE_LIST1: ${FILE_LIST1}

for f in $FILE_LIST1
do 
echo $f
cat $f >> $OUT_FILE
done

echo "set terminal postscript col enhanced 16  " > $GNUPLOTFILE
echo "set border 4095 lt -1 lw 1.000 " >> $GNUPLOTFILE
echo "set samples 5, 5 " >> $GNUPLOTFILE
echo "set isosamples 5, 5 " >> $GNUPLOTFILE
echo "unset surface " >> $GNUPLOTFILE
echo "set style data pm3d " >> $GNUPLOTFILE
echo "set style function pm3d " >> $GNUPLOTFILE
echo "set ticslevel 0 " >> $GNUPLOTFILE
echo "set title \"One-body density\"  font \"\"  " >> $GNUPLOTFILE
echo "set output \"${PS_FILE}\"	 " >> $GNUPLOTFILE
echo "set xrange [ ] noreverse nowriteback	 " >> $GNUPLOTFILE
echo "set xlabel \"x\"  font \"\"		 " >> $GNUPLOTFILE
echo "set ylabel \"t\"  font \"\"		 " >> $GNUPLOTFILE
echo "set yrange [ ] noreverse nowriteback	 " >> $GNUPLOTFILE
echo "set zrange [] noreverse nowriteback	 " >> $GNUPLOTFILE

echo "set pm3d at b			 " >> $GNUPLOTFILE
echo "set view map " >> $GNUPLOTFILE
echo "set pm3d at st			 " >> $GNUPLOTFILE


echo "set pm3d implicit at b " >> $GNUPLOTFILE
echo "set pm3d scansforward " >> $GNUPLOTFILE

echo "set palette rgbformulae 7,5,15 " >> $GNUPLOTFILE
echo "set palette model RGB rgbformulae 7,5,15  " >> $GNUPLOTFILE
echo "set pm3d interpolate 3,1 flush begin ftriangles nohidden3d corners2color mean " >> $GNUPLOTFILE
echo "set size 0.5,1 " >> $GNUPLOTFILE
echo "set size 1.0,1 " >> $GNUPLOTFILE
echo "set origin 0,0 " >> $GNUPLOTFILE

echo "splot [][] \"$OUT_FILE\" using (\$1):(\$5):(\$4) t \" \"   " >> $GNUPLOTFILE

echo "Will now start gnuplot. Have you got it (at least version 4.2) ?"
gnuplot $GNUPLOTFILE
gv $PS_FILE

