!vim:fdm=marker:
!############################################################################################
!#                                                                                          #
!#                 ___                   __  __  ____ _____ ____  _   _ ____                #
!#                / _ ` _ __   ___ _ __ |  `/  |/ ___|_   _|  _ `| | | | __ )               #
!#               | | | | '_ ` / _ ` '_ `| |`/| | |     | | | | | | |_| |  _ `               #
!#               | |_| | |_) |  __/ | | | |  | | |___  | | | |_| |  _  | |_) |              #
!#                `___/| .__/ `___|_| |_|_|  |_|`____| |_| |____/|_| |_|____/               #
!#                     |_|                                                                  #
!#                                      Version 2.3                                         #
!#                                                                                          #
!#Kaspar Sakmann (head of project), Axel Lode, Alexej Streltsov, Ofir Alon, Lorenz Cederbaum#
!#                                                                                          #
!############################################################################################ 
!> \image html OpenMCTDHBlogokaspar.png
!> \mainpage
!> <CENTER> (2007-2012) </CENTER>
!!
!!      <b>OpenMCTDHB </b> is an open-source package for the many-body
!!      dynamics of ultracold bosons [1]. It is an implementation of the MCTDHB 
!! 	algorithm [2] to solve
!!      the many-body Schr&ouml;dinger equation for bosons.
!!      The OpenMCTDHB project was initiated by Kaspar Sakmann 
!!	who also wrote the code up to  Version 2.0  starting in 2007.
!!     
!!      OpenMCTDHB can solve the many-boson Schr&ouml;dinger equation
!!      for 1D, 2D and 3D problems at variable degree of accuracy.
!!      The current version supports up to two orbitals and
!!      thereby allows to assess the validity of  any Gross-Pitaevskii
!!      computation on the many-body level. The code is portable, serial
!!      and can be compiled using open-source software only.
!!      A number of scripts that come with the code allow to analyze the 
!!      physics of  a computation without the need to get deeply involved. 
!!      The goal of OpenMCTDHB is    
!!      to make many-boson dynamics accessible to everyone.
!!      
!!      <HR>
!!     
!! 	People:
!!
!!      - &copy;  2007-2012: Kaspar Sakmann (head of project), Axel U. J. Lode, Alexej 
!!		Streltsov, Ofir Alon, Lorenz Cederbaum
!!
!!      <HR>
!!
!!      A user manual with worked through examples and 
!!      installation instructions can be found at the
!!      <A HREF="http://www.pci.uni-heidelberg.de/tc/usr/mctdhb/OpenMCTDHB.html">
!!      OpenMCTDHB </A> website.
!!      <b> OpenMCTDHB </b> uses some of the libraries of the 
!!      Heidelberg MCTDH package  and comes with a copy of 
!!      the FFTW library (please see the links below).
!!      
!!      If you use <b> OpenMCTDHB </b> (or some results obtained from it) in any
!!      publication, we politely request that you cite [1] below. 
!!      You may also wish to cite the original
!!      MCTDHB paper [2] or the MCTDHB package [3]
!!  
!!      References
!!
!!      [1] OpenMCTDHB v2.3, K. Sakmann, A. U. J. Lode, A. I. Streltsov, O. E. Alon, L. S. Cederbaum (2012), http://OpenMCTDHB.uni-hd.de.
!!
!!      [2] O. E. Alon, A. I. Streltsov, L. S. Cederbaum Phys. Rev. A 77, 033613 (2008).
!!
!!      [3] The MCTDHB Package, A. I. Streltsov, K. Sakmann, A. U. J. Lode,
!!          O. E. Alon, and L. S. Cederbaum, Version 2.0, Heidelberg, (2010), http://MCTDHB.uni-hd.de.
!!
!!      Useful links  
!!      <UL>
!!         <LI>The <A HREF="http://www.pci.uni-heidelberg.de/tc/usr/mctdhb/OpenMCTDHB.html"> 
!!                  OpenMCTDHB </A> website ( downloads, manual, developer guidelines, roadmap, etc. ) 
!!         <LI>The <A HREF="http://www.pci.uni-heidelberg.de/tc/usr/mctdhb/index.html"> 
!!                  MCTDHB </A> website
!!         <LI>The <A HREF="http://www.pci.uni-heidelberg.de/cms/mctdh.html">
!!                  MCTDH </A> website
!!         <LI>The <A HREF="http://www.fftw.org"> FFTW </A> library 
!!      </UL>
!!
!<
!#####################################################################################
!
!>      This is the main program.
!>
!>      Purpose: !       Solve the many-body Schr&ouml;dinger equation for N identical bosons 
!>      
!>       i d/dt |Psi> = H |Psi> 
!> 
!>       using the MCTDHB algorithm. For a thorough description of the algorithm 
!>       see the documentation and the publications.
!>
!>       In short, the MCTDHB algorithm provides the variationally optimal 
!>       solution of the time-dependent many-boson Schr&ouml;dinger equation, given an
!>       ansatz wave function in which N bosons are allowed to reside in M
!>       orbitals. The most striking feature of the MCTDHB algorithm is that
!>       the orbitals in which the many-body wave function is expanded are allowed to
!>       depend on time, and are determined by the principle of least action.
!>       By increasing the number of orbitals, the ansatz for the many-body wave
!>       function becomes more and more general and, therefore, the solution more
!>       accurate. 
!>
!>       If only M=1 orbital is used to expand the many-body wave function,
!>       the MCTDHB equations reduce to the Gross-Pitaevskii equation. MCTDHB can
!>       threfore be considered the variational many-body generalization of the
!>       Gross-Pitaevkii mean-field. The current <b> OpenMCTDHB </b> code supports only 
!>       up to M=2 orbitals. This allows to check whether Gross-Pitaevskii is OK
!>       to use for a given problem.
!>
!>       If the results converge (in regards to increasing the number of orbitals M) 
!>       MCTDHB provides the numerically exact solution of the full
!>       time-dependent many-boson Schr&ouml;dinger equation. To check this it is
!>       generally necessary to use more than M=2 orbitals. However, the current
!>       version of <b> OpenMCTDHB </b> only supports up to M=2 orbitals. More orbitals
!>       are on the road map for <b> OpenMCTDHB </b>.
!>
!>       Stationary states can be obtained by propagation in imaginary time. 
!>
!> \authors Kaspar Sakmann (2007-2012), Axel Lode (2011)
!> \todo Some of the output functions are currently working  for 1D only, make
!>       it 2d. search for 'Dims==1'

PROGRAM OpenMCTDHB

        
  USE   moduleparameters
  USE   moduleinputvariables
  USE   moduledefaults
  USE   modulederivesystemvariables
  USE   moduleallocatables
  USE   moduleinputoutput
  USE   modulecomputationcore
  USE   modulecorrelationfunctions
  USE   modulegrid
  USE   moduleextensions
  USE   moduleintegration

  IMPLICIT NONE
  external AbsABMError

  ! READ INPUT PARAMETERS --------------------------------- 
  ! the file runpars.inp will be read in. the file runpars.out
  ! will be (over)written after every printStep time units 
  ! and can then be used for restarts

  open(110,FILE='runpars.inp',DELIM='APOSTROPHE')
    read(110,NML=SystemParameters)
    read(110,NML=IMESTParameters)
    read(110,NML=GridParameters)
    read(110,NML=PrintParameters)
    read(110,NML=RunParameters)
  close(110)

  ! CONSISTENCY CHECKS OF THE INPUT PARAMETERS -----------------

  CALL check_grid(Dims,NDX,NDY,NDZ)  ! CHECK THE GRID AND DIMENSIONS

  ! INITIALIZE VARIABLES THAT DEPEND ON THE INPUT VARIABLES

  tag = TRIM(ADJUSTL(tag))
  CALL get_nConf(NPar,MOrb,nConf)
  CALL get_nSuperDiags(NPar,MOrb,nSuperDiags) ! for M=1 there are no superdiagonals
  CALL get_totalNoGridPts(NDX,NDY,NDZ,Dims,totalNoGridPts) ! compute the number of grid points
  CALL get_identifier(MOrb,NPar,tag,identifier) ! determine an identifier string for the output files.
  CALL get_weight(xi,xf,yi,yf,zi,zf,NDX,NDY,NDZ,Dims,weight) 

  ! the input variables tinyStep, printStep  and printAllStep control the intervalls 
  ! output files are written:
  ! tinyStep controls scalar output variables that can be monitored continuously,
  ! e.g. the natural orbial occupation numbers.
  ! printNext controls when the one-body density/momentum distribution and a
  ! restart file is written.
  ! printAllNext controls when the first and second order correlation functions
  ! are written.
  printNextTiny   = FLOOR(AbsTime/tinyStep +1.2d0) * tinyStep
  printNext       = FLOOR(AbsTime/printStep + 1.2d0) * printStep
  printAllNext    = FLOOR(AbsTime/printAllStep + 1.2d0) * printAllStep
  
  
  ! PRINT INFO ABOUT THIS RUN TO THE SCREEN
  
  CALL print_header
  CALL print_runInfo(lambda0,NPar,MOrb,Dims,NDX,NDY,NDZ,xi,xf,yi,yf,zi,zf,tinyStep,printStep,printAllStep, &
                     printCorrelationFuncs,AbsTime,maxTime,TolError,restart,Relax,readPotential,propDirection,tag)
  
  
  ! ALLOCATE AND (INITIALIZE TO ZERO) ALL SYSTEM SIZE DEPENDENT ARRAYS --------------------
  ! ADD MORE ALLOCATABLE ARRAYS TO MODULEALLOCATABLES AND MODULEALLOCATE
  
  CALL allocate_systemArrays(NDX,NDY,NDZ,MOrb,nConf,nSuperDiags)
  
  CALL get_p2over2m(NDX,NDY,NDZ,xi,xf,yi,yf,zi,zf,Dims,p2over2m) !--the array p^2/2m for all p values
  CALL get_positionGrid(xi,xf,NDX,gridx) ! the x-grid
  CALL get_positionGrid(yi,yf,NDY,gridy) ! the y-grid
  CALL get_positionGrid(zi,zf,NDZ,gridz) ! the z-grid
  
  ! compute the fourier transform of the interaction potential * (2pi)^0.5 / dV in FFT order
  IF (useIMEST) THEN
    CALL get_vtilde(NDX,NDY,NDZ,Dims,xi,xf,yi,yf,zi,zf,vtilde) 
  END IF
  
 
  IData(1)        =       propDirection   !propagation direction 1 forward, -1 backward
  RData(1)        =       lambda0
  RData(2)        =       TolError 
  LData           =       Relax
  iFail           =       0
  
  !---------------------------------------------------------------------------
  ! Read initial CVec_zero and Psi, or start from a default
  IF (restart) THEN
  
    CALL read_CVecFile(CVec_zero,nConf)
    CALL read_PsiFromFile(Psi_zero,NDX,NDY,NDZ,MOrb)
  
  ELSE 
  
    ! This usually works. All particles start in the first orbital
    CVec_zero     =       ZERO
    CVec_zero(1)  =      (1.d0,0.d0) 
    ! Please set the default initial orbitals in get_defaultOrbitals yourself
    CALL get_defaultOrbitals(NDX,NDY,NDZ,Dims,gridx,gridy,gridz,MOrb,Psi_zero)
  
  END IF

  DO k=1,MOrb
    CALL normvxz(Psi_zero(:,k),norm,NDX*NDY*NDZ) 
    Psi_zero(:,k) = Psi_zero(:,k) / norm 
  END DO

  CALL write_PsiToFile(AbsTime,Psi_zero,NDX,NDY,NDZ,MOrb,0,identifier)
  CALL write_PsiToFile(AbsTime,Psi_zero,NDX,NDY,NDZ,MOrb,1,identifier)

  !--------------------------------------------------------------------------------
  ! This controls what external potential is used. There are two options: the
  ! default is to read in a potential from the file ./restart/V_ext.dat. This
  ! allows to compile the code once and work with different potentials.
  ! However, if a time-dependent external potential is to be used, it is
  ! possible to specify it in the subroutine get_V_ext. The code will have
  ! to be recompiled in this case.
  IF (readPotential) THEN
    
    CALL read_potentialFromFile(V_ext,NDX,NDY,NDZ)
  
  ELSE 
  
    WRITE(*,*) "USE INTERNALLY DEFINED POTENTIAL get_V_ext"
    CALL get_V_ext(V_ext,NDX,NDY,NDZ,Dims,gridx,gridy,gridz,AbsTime)
  
  END IF
  
  !--------------------------------------------------------------------------------
  ! Before the real computation starts, we would like to write out data about 
  ! the initial state. To do that a few quantities need to be computed.
  
  ! Compute the Hamiltonian matrix. 
  CALL  get_FullHamiltonianBandMatrix(NPar,MOrb,NDX,NDY,NDZ,Dims,nConf,&
                                         nSuperDiags,lambda0,Psi_zero,BandMat)
  
  ! Compute the reduced first and second order density matrix elements
  ! and save them in CData, they will be needed in the computation. 
  CALL get_rho_jk(nConf,MOrb,NPar,CVec_zero,rho_jk_zero)
  CALL get_rho_ijkl(nConf,MOrb,NPar,CVec_zero,rho_ijkl_zero)
  CData(:,:,1,1,1) =  rho_jk_zero 
  CData(:,:,:,:,2) =  rho_ijkl_zero
  
  ! compute the chemical potential of the initial guess  
  CALL get_mu_kj(AbsTime,mu_kj,Psi_zero,rho_jk_zero,rho_ijkl_zero)
  CALL write_mu_kj(AbsTime,mu_kj,identifier)
   
  ! Compute the natural orbitals and their occupations, also initialize 
  ! the file that lists the times when a certain fragmentation is
  ! reached for the first time.
  CALL get_naturalOrbitals(MOrb,NDX,NDY,NDZ,rho_jk_zero,Psi_zero,noccs,norbs)
  CALL write_noccsToFile(AbsTime,MOrb,noccs,identifier)
  CALL write_norbsToFile(AbsTime,norbs,NDX,NDY,NDZ,MOrb,identifier)
  CALL write_firstFragmentationTimes(AbsTime,MOrb,NPar,noccs,lambda0,identifier)
  
  
  ! Compute the cumulative density from -infty up to x=0.d0, coded for 1D only so far
  ! and write it to a file. Also the mean density and the Lieb Liniger
  ! parameter are computed
  IF (Dims == 1) THEN
    CALL get_cumulativeProbability(Psi_zero,rho_jk_zero,0.d0,cumulativeProbability)
    CALL write_probabilityLeft(AbsTime,cumulativeProbability,MOrb,NPar,identifier)
  
    CALL get_meanDensity(Psi_zero,rho_jk_zero,meanDensity) 
    CALL write_liebLinigerParameter(AbsTime,lambda0,NPar,meanDensity,identifier)
  END IF
  
  ! Compute the initial one-body density and write it to a file
  ! todo:2d 3d
  CALL get_oneBodyRDMDiagonal(AbsTime,Psi_zero,rho_jk_zero,1,1) 

            !> \todo 3d 
  CALL write_correlations(Dims,printCorrelationFuncs,MOMSPACE2D,REALSPACE2D,&
                          x1const,x1slice,y1const,y1slice,&
                          x2const,x2slice,y2const,y2slice,&
                          AbsTime,Psi_zero,rho_jk_zero,rho_ijkl_zero,&
                          Pnot,xstart,xstop)
 

  ! Write the initial guess to a file
  CALL write_PsiToFile(AbsTime,Psi_zero,NDX,NDY,NDZ,MOrb,0,identifier)
  CALL write_PsiToFile(AbsTime,Psi_zero,NDX,NDY,NDZ,MOrb,1,identifier)
  CALL write_CVecFile(AbsTime,CVec_zero,0,identifier)
  CALL write_CVecFile(AbsTime,CVec_zero,1,identifier)
  
  ! Compute the energy of the initial guess and write it to a file
  CALL get_energy(CVec_zero,BandMat,energy,nConf)
  CALL write_energy(AbsTime,lambda0,NPar,energy,identifier)

  ! integrate the density over some coordinates (sometimes useful for visualization in 2d/3d) 
  ! remove this if you don't want it.
  CALL get_integratedDensity(rho_jk_zero,AbsTime,Psi_zero,writemode=3) !yz integrated out
  CALL get_integratedDensity(rho_jk_zero,AbsTime,Psi_zero,writemode=4) !x integrated out
  CALL get_integratedDensity(rho_jk_zero,AbsTime,Psi_zero,writemode=1) !z integrated out
  CALL get_integratedDensity(rho_jk_zero,AbsTime,Psi_zero,writemode=2) !y integrated out

  !--------------------------------------------------------------------------------
  
  tau             = 0.01d0 
  IntPeriod       = tau
  InitStep        = tau
  Stepsize        = tau ! output of silstep: integration step  performed, can be < IntPeriod 
  Steps           = 0
  RepeatedSteps   = 0 
  IntegratorTolError = 5.0D-1 * TolError 
  
  
  do k=1,MOrb
    CALL normvxz(Psi_zero(:,k),norm,NDX*NDY*NDZ) 
    Psi_zero(:,k) = Psi_zero(:,k) / norm 
    !---{{{error output
       if (ABS(norm-1.d0)>TolError) then 
          write(*,*)k,"norm of initial Psi_zero(:,k) ",norm
       end if 
    !---}}}
  end do
  
  Psi_tau  = Psi_zero  ! initialize Psi_tau for the propagation loop
  CALL normvxz(CVec_zero,norm,nConf)
  CVec_zero = CVec_zero / norm
  !---{{{error output
       if (ABS(norm-1.d0)>TolError) then 
          write(*,*)"norm of initial CVec ",norm
       end if 
  !---}}}

  ! initialize CVec_tau for the propagation loop
  CVec_tau = CVec_zero 
  
  !----------------------------- PROPAGATION LOOP ------------------------------------
  propagation_loop: do 
  
    CVec_zero = CVec_tau
    Psi_zero  = Psi_tau
  
    tau1    = tau
    tau2    = tau
    tau3    = tau
    tausave = tau
    iprintNextTiny       = 0
    iprintNext           = 0
    iprintAllNext        = 0
  
    if (AbsTime+tau > printNextTiny) then
      iprintNextTiny     =       1
      tau1 = printNextTiny-AbsTime
    end if
  
    if (AbsTime+tau > printNext) then
      iprintNextTiny     =       1
      iprintNext         =       1
      tau2 = printNext-AbsTime
    end if
  
    if (AbsTime+tau > printAllNext) then
      iprintNextTiny     =       1
      iprintNext         =       1
      iprintAllNext      =       1
      tau3 = printAllNext-AbsTime
    end if
  
    tau = MIN(ABS(tausave),ABS(tau1),ABS(tau2),ABS(tau3))
    if (ABS(tau) <= 1E-6) tau = 1E-6

    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !+++++++++++++++++++++++++++++++++++ STEP 1 +++++++++++++++++++++++++++++++++++++++++
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    CALL do_stepOne(nConf,MOrb,NPar,CVec_zero,CData,readPotential,V_ext,&
                      NDX,NDY,NDZ,Dims,gridx,gridy,gridz,AbsTime,nSuperDiags,lambda0,&
                      Psi_zero,BandMat,rho_jk_zero,rho_ijkl_zero)

  
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !+++++++++++++++++++++++++++++++++++ STEP 2 +++++++++++++++++++++++++++++++++++++++++
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    CALL do_stepTwo(AbsTime,CVec_zero,DtCVec,BandMat,RData,IData,LData,&
                      CVec_tauhalf,NPar,nConf,MOrb,IntPeriod,SILmaxIntOrder,&
                      IntegratorTolError,Relax,CData,tau, &
                      SILIntPeriod1,nSuperDiags,&
                      rho_ijkl_tauhalf,rho_jk_tauhalf,propDirection)

  
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !+++++++++++++++++++++++++++++++++++ STEP 3 +++++++++++++++++++++++++++++++++++++++++
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 

    CALL do_stepThree(Psi_tauhalf,Psi_zero,Psi_WORK,AbsTime,DtPsi_WORK,&
                        CData,RData,IData,LData,NDX,NDY,NDZ,MOrb,Relax,tau,&
                       IntegratorTolError,restartABM,SILIntPeriod1,&
                       ABMIntOrder) 
    

    !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    propagateif: if (.NOT. Relax) then  ! the following Steps are not needed for Relaxation

      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      !+++++++++++++++++++++++++++++++++++ STEP 4 +++++++++++++++++++++++++++++++++++++++++
      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CALL do_stepFour(rho_jk_zero,rho_ijkl_zero,CData,Rdata,IData,LData,&
                       Psi_tildetauhalf,Psi_zero,restartABM,SILIntPeriod1,&
                       AbsTime,IntegratorTolError,NDX,NDY,NDZ,MOrb,ABMIntOrder)
 
      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      !+++++++++++++++++++++++++++++++++++ STEP 5 +++++++++++++++++++++++++++++++++++++++++
      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CALL do_stepFive(CData,RData,IData,LData,rho_jk_tauhalf,rho_ijkl_tauhalf,Psi_tau,Psi_tauhalf,&
                       readPotential,V_ext,NDX,NDY,NDZ,MOrb,nConf,nPar,Dims,gridx,gridy,gridz,AbsTime,SILIntPeriod1,&
                      ABMIntOrder,IntegratorTolError,restartABM,nSuperDiags,BandMat,lambda0)

      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      !+++++++++++++++++++++++++++++++++++ STEP 6 +++++++++++++++++++++++++++++++++++++++++
      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CALL do_stepSix(CVec_tau,CVec_tauhalf,SILIntPeriod1,BandMat,CData,RData,&
                      IData,LData,nConf,SILmaxIntOrder,IntegratorTolError,Relax,RestartSIL,&
                      StdForm,SILIntPeriod2,AbsTime,iFail,MOrb,nSuperDiags,tau)

      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      !+++++++++++++++++++++++++++++++++++ STEP 7 +++++++++++++++++++++++++++++++++++++++++
      !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CALL do_stepSeven(CVec_tildezero,CVec_tauhalf,AbsTime,SILIntPeriod1,CVec_tau,BandMat,&
                        CData,RData,IData,LData,nConf,MOrb,propDirection,nSuperDiags,SILmaxIntOrder,&
                        IntegratorTolError,Stepsize,iFail,Relax)


 
    end if  propagateif ! Relaxation will continue hereafter
 
    ! make sure CVec_tildezero is normalized
    CALL normvxz(CVec_tildezero,norm,nConf)
    CVec_tildezero = CVec_tildezero / norm
 
    !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !+++++++++++++++++++++++++++++++++++ ERROR EVALUATION +++++++++++++++++++++++++++++++
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
    if (.NOT. Relax) then
 
      ! The error of the propagation is composed of two parts: one from the
      ! propagation of the coefficient vector CVec and one from the propagation of the
      ! orbitals 
      Delta_CVec = CVec_zero - CVec_tildezero
      Delta_Psi  = Psi_tauhalf-Psi_tildetauhalf
 
    else 
 
      CVec_tau = CVec_tauhalf
      Psi_tau  = Psi_tauhalf
      rho_jk_tau = rho_jk_tauhalf
      rho_ijkl_tau = rho_ijkl_tauhalf
 
      IF (.NOT. readPotential) THEN
        CALL get_V_ext(V_ext,NDX,NDY,NDZ,Dims,gridx,gridy,gridz,AbsTime+SILIntPeriod1)
      END IF
 
      CALL  get_FullHamiltonianBandMatrix(NPar,MOrb,NDX,NDY,NDZ,Dims,nConf,&
                                          nSuperDiags,lambda0,Psi_tau,BandMat)
 
      Delta_CVec = CVec_tau - CVec_zero
      Delta_Psi  = Psi_tau - Psi_zero
 
    end if
    
    CALL evaluate_error(Delta_CVec,Delta_Psi,rho_jk_zero,CVecError,PsiError,nConf)
 
    delta = 2*MAX(CVecError,PsiError) 
    if (delta > TolError)  iFail = 3
 
    REJECTSTEPIF: if (iFail.ne.0) then

      CVec_tau = CVec_zero
      Psi_tau  = Psi_zero
      tau = 0.5d0 * tau * (TolError/delta)**0.25
      iFail = 0
 
    else
 
      AbsTime = AbsTime + tau
 
      ! normalize CVec_tau if the step is accepted
      CALL normvxz(CVec_tau,norm,nConf)
      CVec_tau = CVec_tau / norm
 
      ! normalize Psi_tau  if the step is accepted
      do k=1,MOrb
        CALL normvxz(Psi_tau(:,k),norm,NDX*NDY*NDZ) 
        Psi_tau(:,k) = Psi_tau(:,k) / norm 
      end do
      
      ! determine new step size      
      tau = tau * MIN((TolError/delta)**0.25,1.5d0)
 
 
      if (iprintNextTiny==1) then  
 
        CALL get_energy(CVec_tau,BandMat,energy,nConf)
        CALL write_energy(AbsTime,lambda0,NPar,energy,identifier)
 
        if (Relax) then 
          CALL get_defect(nConf,nSuperDiags,BandMat,CVec_tau,energy,defect)
          WRITE(*,1111)"AbsTime:",AbsTime,"  tau:",tau,"  PsiError:",PsiError,&
                                          "  CVecError:",CVecError,"  defect:",defect
          1111 FORMAT(A,F10.4,A,F10.5,A,ES14.6,A,ES14.6,A,ES14.6)
        end if
 
        CALL get_rho_jk(nConf,MOrb,NPar,CVec_tau,rho_jk_tau)
        CALL get_rho_ijkl(nConf,MOrb,NPar,CVec_tau,rho_ijkl_tau)
 
        !> \todo 2d 3d
        IF (Dims==1) THEN
          CALL get_cumulativeProbability(Psi_tau,rho_jk_tau,0.d0,cumulativeProbability)
          CALL write_probabilityLeft(AbsTime,cumulativeProbability,MOrb,NPar,identifier)
        
          CALL get_meanDensity(Psi_tau,rho_jk_tau,meanDensity) 
          CALL write_liebLinigerParameter(AbsTime,lambda0,NPar,meanDensity,identifier)
        END IF 


        CALL get_mu_kj(AbsTime,mu_kj,Psi_tau,rho_jk_tau,rho_ijkl_tau)
        CALL write_mu_kj(AbsTime,mu_kj,identifier)
 
        CALL get_rho_jk(nConf,MOrb,NPar,CVec_tau,rho_jk_tau)
        CALL get_rho_ijkl(nConf,MOrb,NPar,CVec_tau,rho_ijkl_tau)
        CALL get_naturalOrbitals(MOrb,NDX,NDY,NDZ,rho_jk_tau,Psi_tau,noccs,norbs)
        CALL write_noccsToFile(AbsTime,MOrb,noccs,identifier)
 
        CALL write_firstFragmentationTimes(AbsTime,MOrb,NPar,noccs,lambda0,identifier)
        
        printNextTiny = FLOOR(AbsTime/tinyStep + 1.2d0) * tinyStep
        iprintNextTiny = 0
 
        if (iprintNext==1) then
          write(*,*)"AbsTime = ",AbsTime
          open(111,FILE='runpars.out',DELIM='APOSTROPHE')
            write(UNIT=111,NML=SystemParameters)
            write(UNIT=111,NML=IMESTParameters)
            write(UNIT=111,NML=GridParameters)
            write(UNIT=111,NML=PrintParameters)
            write(UNIT=111,NML=RunParameters)
          close(111)
          
          CALL write_PsiToFile(AbsTime,Psi_tau,NDX,NDY,NDZ,MOrb,1,identifier)
          CALL write_CVecFile(AbsTime,CVec_tau,1,identifier)
          CALL get_oneBodyRDMDiagonal(AbsTime,Psi_tau,rho_jk_tau,1,1) !-write 1B density 
 
          !> \todo 2d 3d 
          IF(Dims==1) THEN
            CALL pedestrian_FT(Psi_tau,FTPsi,1)
            CALL get_dilation(dilation,FTPsi,rho_jk_tau)
            CALL pedestrian_FT(Psi_tau,FTPsi,dilation)
            CALL get_oneBodyRDMDiagonal(AbsTime,FTPsi,rho_jk_tau,2,dilation) !-write 1B momentum distribution
          END IF
    
          CALL get_rho_jk(nConf,MOrb,NPar,CVec_tau,rho_jk_tau)
          CALL get_rho_ijkl(nConf,MOrb,NPar,CVec_tau,rho_ijkl_tau)
          CALL get_naturalOrbitals(MOrb,NDX,NDY,NDZ,rho_jk_tau,Psi_tau,noccs,norbs)
          CALL write_noccsToFile(AbsTime,MOrb,noccs,identifier)
  
          printNext = FLOOR(AbsTime/printStep + 1.2d0) * printStep
          iprintNext = 0
 
          if (iprintAllNext==1) then
 
            CALL write_PsiToFile(AbsTime,Psi_tau,NDX,NDY,NDZ,MOrb,0,identifier)
            CALL write_CVecFile(AbsTime,CVec_tau,0,identifier)
            CALL write_norbsToFile(AbsTime,norbs,NDX,NDY,NDZ,MOrb,identifier)
    
            CALL write_correlations(Dims,printCorrelationFuncs,MOMSPACE2D,REALSPACE2D,&
                                    x1const,x1slice,y1const,y1slice,&
                                    x2const,x2slice,y2const,y2slice,&
                                    AbsTime,Psi_tau,rho_jk_tau,rho_ijkl_tau,&
                                    Pnot,xstart,xstop)
                                       
           iprintAllNext = 0
            printAllNext  = FLOOR(AbsTime/printAllStep + 1.2d0) * printAllStep
 
          end if
 
        end if 
 
        tau = MAX(tausave,tau)
 
      end if 
        
    end if REJECTSTEPIF
 
    if (AbsTime.ge.maxTime) then
       write(*,*)" maxtime reached"   
       exit propagation_loop
    end if
    
 
  end do propagation_loop

  CALL get_rho_jk(nConf,MOrb,NPar,CVec_tau,rho_jk_tau)
  CALL get_rho_ijkl(nConf,MOrb,NPar,CVec_tau,rho_ijkl_tau)
  CALL get_naturalOrbitals(MOrb,NDX,NDY,NDZ,rho_jk_tau,Psi_tau,noccs,norbs)
  CALL write_norbsToFile(AbsTime,norbs,NDX,NDY,NDZ,MOrb,identifier)
  CALL write_noccsToFile(AbsTime,MOrb,noccs,identifier)
  CALL write_firstFragmentationTimes(AbsTime,MOrb,NPar,noccs,lambda0,identifier)

  CALL write_PsiToFile(AbsTime,Psi_tau,NDX,NDY,NDZ,MOrb,0,identifier)
  CALL write_PsiToFile(AbsTime,Psi_tau,NDX,NDY,NDZ,MOrb,1,identifier)
  CALL write_CVecFile(AbsTime,CVec_tau,0,identifier)
  CALL write_CVecFile(AbsTime,CVec_tau,1,identifier)

  ! integrate the density over some coordinates (sometimes useful for visualization in 2d/3d) 
  ! remove this if you don't want it.
  CALL get_integratedDensity(rho_jk_tau,AbsTime,Psi_tau,writemode=3)   !yz integrated out
  CALL get_integratedDensity(rho_jk_tau,AbsTime,Psi_tau,writemode=4)   !x integrated out
  CALL get_integratedDensity(rho_jk_tau,AbsTime,Psi_tau,writemode=1)   !z integrated out
  CALL get_integratedDensity(rho_jk_tau,AbsTime,Psi_tau,writemode=2)   !y integrated out


  !>\todo 2d 3d 
  IF (Dims == 1) THEN
    CALL get_oneBodyRDMDiagonal(AbsTime,Psi_tau,rho_jk_tau,1,1) !-write 1B density 
  END IF

  CALL deallocate_systemArrays 

  write(*,*)"END OF PROGRAM REACHED, ADIEU."

end program OpenMCTDHB


