! vim:fdm=marker:

!> This module contains declarations of all allocatable variables and the (de)allocate subroutines
!>
!> \author Kaspar Sakmann (2012)
!> \since 2007
!> \todo document variables of this module
MODULE moduleallocatables
IMPLICIT NONE

!------------------- FIRST AND SECOND ORDER DENSITY MATRICES AT DIFFERENT TIMES
COMPLEX*16,ALLOCATABLE,DIMENSION(:,:)           :: rho_jk_zero,rho_jk_tauhalf,rho_jk_tau
COMPLEX*16,ALLOCATABLE,DIMENSION(:,:,:,:)       :: rho_ijkl_zero,rho_ijkl_tauhalf,rho_ijkl_tau



!------------------- THE ABM WORKARRAY, ORBITALS, ETC. ----------------------------------------------
COMPLEX*16,ALLOCATABLE,DIMENSION(:,:)           :: AuxPsi     ! at least (totalNoGridPts,ABMIntOrder+1)
COMPLEX*16, ALLOCATABLE, DIMENSION(:)           :: Psi_WORK,DtPsi_WORK
COMPLEX*16, ALLOCATABLE, DIMENSION(:,:)         :: Psi_zero,Psi_tauhalf,&
                                                   Psi_tildetauhalf,Psi_tau,DtPsiCheck,DtPsi,&
                                                   FTPsi,norbs,Delta_Psi,diffPsi,&
                                                   Chi,projectedChi
REAL*8, ALLOCATABLE,DIMENSION(:)                :: noccs


!------------------- SIL VECTORS, HAMILTONIAN MATRIX, ETC.------------------------------------------
complex*16,ALLOCATABLE,DIMENSION(:)             :: DtCVec,CVec_zero,CVec_tauhalf,CVec_tau, &
                                                   CVec_tildezero,Delta_CVec
complex*16,ALLOCATABLE,DIMENSION(:,:)           :: BandMat,Krylov 

COMPLEX*16,ALLOCATABLE, DIMENSION(:,:)          :: mu_kj


!------------------- DATA ARRAY THAT IS PASSED TO THE INTEGRATORS --------------------------------
COMPLEX*16,ALLOCATABLE,DIMENSION(:,:,:,:,:)     :: CData 


!------------------- EXTERNAL POTENTIAL
REAL*8, ALLOCATABLE,DIMENSION(:) :: V_ext,p2over2m
REAL*8, ALLOCATABLE,DIMENSION(:) :: gridx,gridy,gridz

!------------------- INTERACTION POTENTIAL
COMPLEX*16, ALLOCATABLE,DIMENSION(:) :: vtilde


CONTAINS


!---------------------------------------------------------------------------------------------------
!> This routine allocates all arrays that depend on the input parameters, e.g.,
!> the array containing the orbitals Psi, the vector containing the coefficients
!> CVec, etc.
!>
!> \authors Kaspar Sakmann (2011)
!> \since 2007
!> \todo maybe get rid of the use association
SUBROUTINE allocate_systemArrays(NDX,NDY,NDZ,MOrb,nConf,nSuperDiags)
USE moduleparameters,ONLY:ABMIntOrder,SILmaxIntOrder,ZERO
!---{{{
IMPLICIT NONE

INTEGER,INTENT(IN)      :: NDX,NDY,NDZ,MOrb,nConf,nSuperDiags
INTEGER                 :: status=0

WRITE(*,100)'--> Allocating system size dependent arrays'
ALLOCATE(Psi_WORK(NDX*NDY*NDZ*MOrb),DtPsi_WORK(NDX*NDY*NDZ*MOrb),STAT=status)

IF (status == 0) THEN 
  Psi_WORK                =       ZERO
  DtPsi_WORK              =       ZERO
ELSE
  WRITE(*,100)'ARRAY ALLOCATION FAILURE 1'
  STOP
END IF


ALLOCATE(Psi_zero(NDX*NDY*NDZ,MOrb),&
         Psi_tauhalf(NDX*NDY*NDZ,MOrb),Psi_tildetauhalf(NDX*NDY*NDZ,MOrb),Psi_tau(NDX*NDY*NDZ,MOrb),&
         DtPsiCheck(NDX*NDY*NDZ,MOrb),DtPsi(NDX*NDY*NDZ,MOrb),FTPsi(NDX*NDY*NDZ,MOrb),&
         norbs(NDX*NDY*NDZ,MOrb),noccs(MOrb),Delta_Psi(NDX*NDY*NDZ,MOrb),diffPsi(NDX*NDY*NDZ,MOrb),&
         Chi(NDX*NDY*NDZ,MOrb),projectedChi(NDX*NDY*NDZ,MOrb), AuxPsi(NDX*NDY*NDZ*MOrb,ABMIntOrder+1),&
         STAT=status)   

IF (status == 0) THEN 
  Psi_zero                =       ZERO 
  Psi_tauhalf             =       ZERO 
  Psi_tildetauhalf        =       ZERO 
  Psi_tau                 =       ZERO 
  DtPsiCheck              =       ZERO 
  DtPsi                   =       ZERO 
  FTPsi                   =       ZERO 
  norbs                   =       ZERO 
  noccs                   =       0.0d0
  Delta_Psi               =       ZERO 
  diffPsi                 =       ZERO 
  Chi                     =       ZERO 
  projectedChi            =       ZERO 
  AuxPsi                  =       ZERO 
ELSE
  WRITE(*,100)'ARRAY ALLOCATION FAILURE 2'
  STOP
END IF


ALLOCATE(CData(MOrb,MOrb,MOrb,MOrb,2),rho_jk_zero(MOrb,MOrb),rho_ijkl_zero(MOrb,MOrb,MOrb,MOrb),&
         rho_jk_tauhalf(MOrb,MOrb),rho_ijkl_tauhalf(MOrb,MOrb,MOrb,MOrb),&
         rho_jk_tau(MOrb,MOrb),rho_ijkl_tau(MOrb,MOrb,MOrb,MOrb), STAT=status)

IF (status == 0) THEN 
  CData                 =       ZERO
  rho_jk_zero           =       ZERO
  rho_ijkl_zero         =       ZERO
  rho_jk_tauhalf        =       ZERO
  rho_ijkl_tauhalf      =       ZERO
  rho_jk_tau            =       ZERO
  rho_ijkl_tau          =       ZERO
ELSE
  WRITE(*,100)'ARRAY ALLOCATION FAILURE 3'
  STOP
END IF


ALLOCATE(DtCVec(nConf),CVec_zero(nConf),CVec_tauhalf(nConf),CVec_tau(nConf), &
         CVec_tildezero(nConf),Delta_CVec(nConf),BandMat(nSuperDiags+1,nConf),&
         Krylov(nConf,2:SILmaxIntOrder),mu_kj(MOrb,MOrb),STAT=status)

IF (status == 0) THEN 
  DtCVec                =       ZERO
  CVec_zero             =       ZERO
  CVec_tauhalf          =       ZERO
  CVec_tau              =       ZERO
  CVec_tildezero        =       ZERO
  Delta_CVec            =       ZERO
  BandMat               =       ZERO
  Krylov                =       ZERO
  mu_kj                 =       ZERO
ELSE
  WRITE(*,100)'ARRAY ALLOCATION FAILURE 4'
  STOP
END IF

ALLOCATE(V_ext(NDX*NDY*NDZ),p2over2m(NDX*NDY*NDZ),vtilde(NDX*NDY*NDZ),&
         gridx(NDX),gridy(NDY),gridz(NDZ),STAT=status)


IF (status == 0) THEN 
  V_ext         =       0.0d0
  p2over2m      =       0.0d0
  vtilde        =       (0.0d0,0.d0)
  gridx =       0.0d0
  gridy =       0.0d0
  gridz =       0.0d0
ELSE
  WRITE(*,100)'ARRAY ALLOCATION FAILURE 5'
  STOP
END IF

!WRITE(*,100)'<-- Allocation successful'

100 FORMAT (5X,A)
!---}}}
END SUBROUTINE allocate_systemArrays

!---------------------------------------------------------------------------------------------------
!> This routine deallocates all allocatable arrays allocated by allocate_systemArrays
!>
!> \authors Kaspar Sakmann (2011)
!> \since 2007
SUBROUTINE deallocate_systemArrays
!---{{{
IMPLICIT NONE

INTEGER                 :: status

WRITE(*,100)'--> Deallocating system size dependent arrays'
DEALLOCATE(Psi_WORK,DtPsi_WORK,STAT=status)

IF (status /= 0) THEN 
  WRITE(*,100)'ARRAY DEALLOCATION FAILURE 1'
  STOP
END IF

DEALLOCATE(Psi_zero, Psi_tauhalf,Psi_tildetauhalf,Psi_tau,&
         DtPsiCheck,DtPsi,FTPsi,norbs,noccs,Delta_Psi,diffPsi,&
         Chi,projectedChi, STAT=status)   

IF (status /= 0) THEN 
  WRITE(*,100)'ARRAY DEALLOCATION FAILURE 2'
  STOP
END IF

DEALLOCATE(rho_jk_zero,rho_jk_tauhalf,rho_jk_tau, &
           rho_ijkl_zero,rho_ijkl_tauhalf,rho_ijkl_tau,&
           CData, STAT=status)


IF (status /= 0) THEN 
  WRITE(*,100)'ARRAY DEALLOCATION FAILURE 3'
  STOP
END IF

DEALLOCATE(DtCVec,CVec_zero,CVec_tauhalf,CVec_tau, &
           CVec_tildezero,Delta_CVec,BandMat,&
           Krylov,STAT=status)


IF (status /= 0) THEN 
  WRITE(*,100)'ARRAY DEALLOCATION FAILURE 4'
  STOP
END IF

DEALLOCATE(V_ext,p2over2m,vtilde,gridx,gridy,gridz,STAT=status)


IF (status /= 0) THEN 
  WRITE(*,100)'ARRAY DEALLOCATION FAILURE 5'
  STOP
END IF

!WRITE(*,100)'<-- Deallocation successful'

100 FORMAT (5X,A)
!---}}}
END SUBROUTINE deallocate_systemArrays


END MODULE moduleallocatables
